#include <windows.h>
#include <nt\commctrl.h>
#include <math.h>

#define REDCHECK 201
#define GRNCHECK 202
#define BLUCHECK 203
#define APPLYCHECK 204
#define SINWAVE 301
#define SQRWAVE 302
#define SAWWAVE 303
#define DISTXTRACK 401
#define DISTYTRACK 402
#define POWERTRACK 403
#define IDAPPLY 500

char progname[]="Plasma generator";
char SINTAB[256];
HINSTANCE hInst;

struct configstruct {
  int red,green,blue;
  int autoapply;
  int wavetype;
  int distx;
  int disty;
  int power;  
};

struct configstruct config;

void genwave(int wavetype) {
  int i;
  switch (wavetype) {
  case 1:
    for (i=0;i<256;i++)
      SINTAB[i]=sin(((i+1)*3.14159265359)/128)*127+128;
    break;
  case 2:
    for (i=0;i<256;i++)
      SINTAB[i]=(i<127)?64:192;
    break;
  case 3:
    for (i=0;i<256;i++)
      SINTAB[i]=i;
    break;
  }
}

// forward declaration:
LRESULT CALLBACK winproc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInst, LPSTR lpszCmdLine, int nCmdShow)
{
  WNDCLASSEX winclass;
  HWND hWnd;
  MSG msg;
  hInst=hInstance;

  winclass.cbSize=sizeof(WNDCLASSEX);
  winclass.style=CS_DBLCLKS;
  winclass.lpfnWndProc=&winproc;
  winclass.cbClsExtra=0;
  winclass.cbWndExtra=0;
  winclass.hInstance=hInst;
  winclass.hIcon=LoadIcon(hInst,"OURICON");
  winclass.hCursor=LoadCursor(NULL,IDC_ARROW);
  winclass.hbrBackground=NULL;
  winclass.lpszMenuName="OURMENU";
  winclass.lpszClassName=progname;
  winclass.hIconSm=NULL;

  if (!RegisterClassEx(&winclass))
    return 0;

  genwave(1);

  config.red=1;
  config.green=1;
  config.blue=1;
  config.autoapply=0;
  config.wavetype=1;
  config.distx=0;
  config.disty=0;
  config.power=0;  


  InitCommonControls();

  hWnd=CreateWindow(
    progname,
    progname,
    WS_SYSMENU|WS_CAPTION|WS_BORDER|WS_OVERLAPPED|WS_VISIBLE|WS_MINIMIZEBOX,
    CW_USEDEFAULT,
    0,
    320+2,
    200+16+16+2,
    NULL,
    NULL,
    hInst,
    NULL);
  ShowWindow(hWnd,nCmdShow);
  UpdateWindow(hWnd);
  while (GetMessage(&msg,NULL,0,0)) 
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
  return (msg.wParam);
}

HDC pDC;
HBITMAP old;
HBITMAP ourbitmap;
int * framebuf;

int gen_red(int tick,int x, int y) {
  if (config.red) {
    return SINTAB[(y/4-x+tick)&0xff];
  } else {
    return 0;
  }
}

int gen_green(int tick,int x, int y) {
  if (config.green) {
    return SINTAB[(SINTAB[(x+y)&0xff]-SINTAB[tick&0xff])&0xff];
  } else {
    return 0;
  }
}

int gen_blue(int tick,int x, int y) {
  if (config.blue) {
    return SINTAB[(x/4-y+tick)&0xff];
  } else {
    return 0;
  }
}


void render_effect(int tick,int * framebuf)
{
  int i,j,k;
  int xrip[200],yrip[320];  
  tick/=16;
  for (i=0;i<200;i++)
    xrip[i]=(((SINTAB[(i*4+tick)&0xff]*(16+config.power))>>4)*config.distx)>>2;
  for (i=0;i<320;i++)
    yrip[i]=(((SINTAB[(i*4-tick)&0xff]*(16+config.power))>>4)*config.disty)>>2;
  for (k=0,i=0;i<200;i++)
    for (j=0;j<320;j++,k++)
      *(framebuf+k)=RGB(gen_blue(tick,yrip[j]+(i*(16+config.power))>>4,xrip[i]+(j*(16+config.power))>>4),
                        gen_green(tick,yrip[j]+(i*(16+config.power))>>4,xrip[i]+(j*(16+config.power))>>4),
                        gen_red(tick,yrip[j]+(i*(16+config.power))>>4,xrip[i]+(j*(16+config.power))>>4));
}

void render(HDC hDC)
{
  render_effect(GetTickCount(),framebuf);
  BitBlt(hDC,0,0,320,200,pDC,0,0,SRCCOPY);
}

void deinit_framebuf(void)
{
  SelectObject(pDC,old);
  DeleteDC(pDC);
  DeleteObject(ourbitmap);
}

void init_framebuf(void)
{
  HDC hDC;
  BITMAPINFO bitmapinfo;
  hDC=CreateCompatibleDC(NULL);
  bitmapinfo.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
  bitmapinfo.bmiHeader.biWidth=320;
  bitmapinfo.bmiHeader.biHeight=-200; /* top-down */
  bitmapinfo.bmiHeader.biPlanes=1;
  bitmapinfo.bmiHeader.biBitCount=32;
  bitmapinfo.bmiHeader.biCompression=BI_RGB;
  bitmapinfo.bmiHeader.biSizeImage=0;
  bitmapinfo.bmiHeader.biClrUsed=256;
  bitmapinfo.bmiHeader.biClrImportant=256;
  ourbitmap=CreateDIBSection(hDC,&bitmapinfo,DIB_RGB_COLORS,&framebuf,0,0);
  pDC=CreateCompatibleDC(NULL);
  old=SelectObject(pDC,ourbitmap);
  DeleteDC(hDC);
}

struct configstruct oldconfig, tempconfig;

BOOL APIENTRY ControlProc(HWND hDlg,UINT uiMessage,UINT wParam,LONG lParam) {
  switch (uiMessage) {
  case WM_HSCROLL:
  case WM_VSCROLL:
    wParam=0;
  case WM_COMMAND:
    tempconfig.distx=SendDlgItemMessage(hDlg,DISTXTRACK,TBM_GETPOS,0,0);
    tempconfig.disty=SendDlgItemMessage(hDlg,DISTYTRACK,TBM_GETPOS,0,0);
    tempconfig.power=SendDlgItemMessage(hDlg,POWERTRACK,TBM_GETPOS,0,0);
    tempconfig.autoapply=IsDlgButtonChecked(hDlg,APPLYCHECK);
    tempconfig.red=IsDlgButtonChecked(hDlg,REDCHECK);
    tempconfig.green=IsDlgButtonChecked(hDlg,GRNCHECK);
    tempconfig.blue=IsDlgButtonChecked(hDlg,BLUCHECK);
    if (IsDlgButtonChecked(hDlg,SINWAVE))
      tempconfig.wavetype=1;
    if (IsDlgButtonChecked(hDlg,SQRWAVE))
      tempconfig.wavetype=2;
    if (IsDlgButtonChecked(hDlg,SAWWAVE))
      tempconfig.wavetype=3;
    if (LOWORD(wParam)==IDOK) {
      memcpy((char*)&config,(char*)&tempconfig,sizeof(struct configstruct));
      EndDialog(hDlg,TRUE);
      genwave(config.wavetype);
      return FALSE;
    }
    if ((LOWORD(wParam)==IDAPPLY) || tempconfig.autoapply) {
      memcpy((char*)&config,(char*)&tempconfig,sizeof(struct configstruct));
      genwave(config.wavetype);
    }
    if (LOWORD(wParam)==IDCANCEL) {
      memcpy((char*)&config,(char*)&oldconfig,sizeof(struct configstruct));
      EndDialog(hDlg,TRUE);
      genwave(config.wavetype);
      return FALSE;
    }      
    break;
  case WM_INITDIALOG:
    memcpy((char*)&oldconfig,(char*)&config,sizeof(struct configstruct));
    memcpy((char*)&tempconfig,(char*)&config,sizeof(struct configstruct));
    SendDlgItemMessage(hDlg,DISTXTRACK,TBM_SETRANGE,TRUE,MAKELONG(0,30));
    SendDlgItemMessage(hDlg,DISTYTRACK,TBM_SETRANGE,TRUE,MAKELONG(0,30));
    SendDlgItemMessage(hDlg,POWERTRACK,TBM_SETRANGE,TRUE,MAKELONG(0,62));
    SendDlgItemMessage(hDlg,DISTXTRACK,TBM_SETPOS,1,config.distx);
    SendDlgItemMessage(hDlg,DISTYTRACK,TBM_SETPOS,1,config.disty);
    SendDlgItemMessage(hDlg,POWERTRACK,TBM_SETPOS,1,config.power);
    CheckDlgButton(hDlg,APPLYCHECK,config.autoapply);
    CheckDlgButton(hDlg,REDCHECK,config.red);
    CheckDlgButton(hDlg,GRNCHECK,config.green);
    CheckDlgButton(hDlg,BLUCHECK,config.blue);
    switch (config.wavetype) {
    case 1:CheckDlgButton(hDlg,SINWAVE,1); break;
    case 2:CheckDlgButton(hDlg,SQRWAVE,1); break;
    case 3:CheckDlgButton(hDlg,SAWWAVE,1); break;
    }
    
    break;
  }
  return FALSE;
}

LRESULT CALLBACK winproc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  HDC hDC;
  PAINTSTRUCT PtStr;
  switch (uMsg) {
  case WM_COMMAND:
    switch (LOWORD(wParam)) {
    case 1001:
      MessageBox(hWnd,"Plasma generator 0.01a","Sorry, no about dialog =)",MB_OK|MB_ICONEXCLAMATION);
      break;
    case 1002:
      SendMessage(hWnd,WM_CLOSE,0,0);
      break;
    case 1101:
      /* display controls */
      DialogBox(hInst,"OURDIALOG",hWnd,(DLGPROC)&ControlProc);
      break;
    }
    break;    
  case WM_DESTROY:
    deinit_framebuf();
    PostQuitMessage(0);
    KillTimer (hWnd, 1);
    break;
  case WM_CREATE:
    SetTimer (hWnd, 1, 10, NULL);
    init_framebuf();
    break;
  case WM_TIMER:
    InvalidateRgn(hWnd,0,0);
    UpdateWindow (hWnd);
    break;
  case WM_PAINT:
    hDC=BeginPaint(hWnd,&PtStr);
    render(hDC);
    EndPaint(hWnd,&PtStr);
    break;
  default:
    return DefWindowProc (hWnd, uMsg, wParam, lParam);
    break;
  }
  return 0;
}
